/*
Hijackulator - ZX Spectrum emulator with support for native-code enhancements
Copyright (C) 2003 Matthew Westcott

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Some portions of this software originate from the JEMU project:
http://jemu.emuunlim.com/
*/

/*
 * Hijackulator.java
 *
 * Created on 27 April 2003, 15:05
 *
 */

package org.zxdemo.hijackulator;

import java.io.*;
import java.net.URL;

/**
 * Main Hijackulator applet
 * @author  Matthew Westcott
 */
public class Hijackulator extends java.applet.Applet implements java.lang.Runnable {
    
    protected Z80 processor = new Z80(3500000);
    protected Speccy48Memory memory = new Speccy48Memory();
    protected SpeccyKeyboard keyboard = new SpeccyKeyboard();
    protected Trap[] traps = new Trap[65536];
    protected SpeccyScreen screen;
    protected boolean isQuitting = false;
    protected Thread processorThread;
    
    /** Initialization method that will be called after the applet is loaded
     *  into the browser.
     */
    public void init() {
        this.resize(512,384);
        //this.resize(512,512);
        
        byte[] romData = new byte[16384];
        URL romUrl;
        URL snapUrl;
        
        try {
            romUrl = new URL( getDocumentBase(), "spectrum.rom" );
        } catch (Exception e) {
            System.err.println("Something wrong with ROM URL");
            return;
        }

        try {
            DataInputStream romStream = new DataInputStream(romUrl.openStream());
            romStream.readFully(romData);
        } catch (IOException e) {
            System.err.println("Miscellaneous IO error loading ROM file");
        }
        
        try {
            snapUrl = new URL( getDocumentBase(), "hskiing.sna");
        } catch (Exception e) {
            System.err.println("Something wrong with snapshot URL");
            return;
        }
        
        this.memory.setMemory(0,romData);
        this.processor.setMemoryDevice(this.memory);
        this.processor.setInterrupDevice(new TrivialInterruptDevice(this.processor));
        this.screen = new SpeccyScreen(getGraphics(), this.memory);
        // this.screen.processor = this.processor;
        this.memory.setDisplay(this.screen);
        
        // attach keyboard
        this.addKeyListener(this.keyboard);
        this.processor.addInputDeviceMapping(
            new DeviceMapping(this.keyboard, 0x0001, 0x0000)
        );
        
        // load snapshot
        Snapshot snap = new Snapshot(snapUrl, this.memory);
        this.processor.setSnapshotState(snap);
        // this.memory.setMemory(16384,snap.memoryDump);
        
        setTraps(new HoraceGoesSkiingHijack(this));
        
        this.processorThread = new Thread(this);
        this.processorThread.start();
    }
    
    private void setTraps(HijackDefinition hijack) {
        Trap[] hijackTraps = hijack.getTraps();
        for (int i=0; i < hijackTraps.length; i++) {
            this.traps[hijackTraps[i].getAddress()] = hijackTraps[i];
            hijackTraps[i].setParent(hijack);
        }
    }
    
    public void stop() {
        this.isQuitting = true;
        this.processorThread.interrupt();
    }
    
    public void run() {
        while (!this.isQuitting) {
            for(int i=0; i < 69888; i++) {
                this.processor.step();
                if (this.traps[this.processor.getProgramCounter()] != null) {
                    this.traps[this.processor.getProgramCounter()].hijack (
                        processor, memory, screen
                    );
                }
//                if (this.processor.getProgramCounter() == 0x6c20) {
//                    this.screen.pen = false;
//                }
                
//                if (this.processor.getProgramCounter() == 0x6c8a) {
//                    this.screen.pen = true;
//                }

                
//                System.err.println("PC:" + this.processor.getProgramCounter());
            }
//            this.showStatus("PC:" + this.processor.getProgramCounter());
//            this.showStatus(this.processor.getState());
            this.processor.setInterruptPending(true);
            try {
                this.processorThread.sleep(20);
            } catch (InterruptedException e) {
            }
        }
    }
}
