/*
Hijackulator - ZX Spectrum emulator with support for native-code enhancements
Copyright (C) 2003 Matthew Westcott

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Some portions of this software originate from the JEMU project:
http://jemu.emuunlim.com/
*/

/*
 * Snapshot.java
 *
 * Created on 29 April 2003, 23:15
 */

package org.zxdemo.hijackulator;

import java.io.*;
import java.net.URL;

/**
 * Emulator-friendly representation of a snapshot
 *
 * @author  Matthew Westcott
 */
public class Snapshot {
    
    public int af = 0; public int bc = 0; public int de = 0; public int hl = 0;
    public int af1 = 0; public int bc1 = 0; public int de1 = 0; public int hl1 = 0;
    public boolean iff1 = false; public boolean iff2 = false;
    public int im = 0;
    public int ix = 0; public int iy = 0;
    public int pc = 0; public int sp = 0;
    public int i = 0; public int r = 0;
    
    public byte[] memoryDump = new byte[49152];
    
    /** Creates a new instance of Snapshot */
    public Snapshot(URL url, Speccy48Memory mem) {
        // only 48k SNA handled for now
        try {
            DataInputStream snapData = new DataInputStream(url.openStream());
            this.i = snapData.readUnsignedByte();
            this.hl1 = readLittleEndianWord(snapData);
            this.de1 = readLittleEndianWord(snapData);
            this.bc1 = readLittleEndianWord(snapData);
            this.af1 = readLittleEndianWord(snapData);
            this.hl = readLittleEndianWord(snapData);
            this.de = readLittleEndianWord(snapData);
            this.bc = readLittleEndianWord(snapData);
            this.iy = readLittleEndianWord(snapData);
            this.ix = readLittleEndianWord(snapData);
            this.iff2 = this.iff1 = (snapData.readUnsignedByte() & 0x04) != 0;
            this.r = snapData.readUnsignedByte();
            this.af = readLittleEndianWord(snapData);
            this.sp = readLittleEndianWord(snapData);
            this.im = snapData.readUnsignedByte();
            snapData.readUnsignedByte(); // throw away border colour
            snapData.readFully(memoryDump);
            
            mem.setMemory(16384,this.memoryDump);
            
            this.pc = (mem.readByte(this.sp))
                | (mem.readByte(this.sp + 1) << 8);
            mem.writeByte(this.sp, 0);
            mem.writeByte(this.sp + 1, 0);
            this.sp += 2;
        } catch (Exception e) {
            // no big deal if it fails
            System.err.println("Oh no! "+e);
        }
    }
    
    private int readLittleEndianWord(DataInputStream stream) throws IOException {
        int lo = stream.readUnsignedByte();
        int hi = stream.readUnsignedByte();
        int res = (hi << 8) | lo;
        return res;
    }
    
}
