/*
Hijackulator - ZX Spectrum emulator with support for native-code enhancements
Copyright (C) 2003 Matthew Westcott

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Some portions of this software originate from the JEMU project:
http://jemu.emuunlim.com/
*/

/*
 * SpeccyKeyboard.java
 *
 * Created on 29 April 2003, 20:53
 */

package org.zxdemo.hijackulator;

import java.awt.event.*;

/**
 * Spectrum-specific keyboard handling routines
 * @author  Matthew Westcott
 */
public class SpeccyKeyboard extends Device implements KeyListener {
    
    protected int[] keyRowState = new int[8];
    protected int[] keyRow = new int[256];
    protected int[] keyBit = new int[256];
    
    /** Creates a new instance of SpeccyKeyboard */
    public SpeccyKeyboard() {
        super("Speccy keyboard (sproing!)");
        for (int i=0; i < 8; i++) {
            this.keyRowState[i] = 0x1F;
        }
        
        // keyboard mappings: keycode -> addressed row, keycode -> bit
        keyRow[KeyEvent.VK_SHIFT]    = 0;    keyBit[KeyEvent.VK_SHIFT]    = 0x01;
        keyRow[KeyEvent.VK_Z]        = 0;    keyBit[KeyEvent.VK_Z]        = 0x02;
        keyRow[KeyEvent.VK_X]        = 0;    keyBit[KeyEvent.VK_X]        = 0x04;
        keyRow[KeyEvent.VK_C]        = 0;    keyBit[KeyEvent.VK_C]        = 0x08;
        keyRow[KeyEvent.VK_V]        = 0;    keyBit[KeyEvent.VK_V]        = 0x10;
        keyRow[KeyEvent.VK_A]        = 1;    keyBit[KeyEvent.VK_A]        = 0x01;
        keyRow[KeyEvent.VK_S]        = 1;    keyBit[KeyEvent.VK_S]        = 0x02;
        keyRow[KeyEvent.VK_D]        = 1;    keyBit[KeyEvent.VK_D]        = 0x04;
        keyRow[KeyEvent.VK_F]        = 1;    keyBit[KeyEvent.VK_F]        = 0x08;
        keyRow[KeyEvent.VK_G]        = 1;    keyBit[KeyEvent.VK_G]        = 0x10;
        keyRow[KeyEvent.VK_Q]        = 2;    keyBit[KeyEvent.VK_Q]        = 0x01;
        keyRow[KeyEvent.VK_W]        = 2;    keyBit[KeyEvent.VK_W]        = 0x02;
        keyRow[KeyEvent.VK_E]        = 2;    keyBit[KeyEvent.VK_E]        = 0x04;
        keyRow[KeyEvent.VK_R]        = 2;    keyBit[KeyEvent.VK_R]        = 0x08;
        keyRow[KeyEvent.VK_T]        = 2;    keyBit[KeyEvent.VK_T]        = 0x10;
        keyRow[KeyEvent.VK_1]        = 3;    keyBit[KeyEvent.VK_1]        = 0x01;
        keyRow[KeyEvent.VK_2]        = 3;    keyBit[KeyEvent.VK_2]        = 0x02;
        keyRow[KeyEvent.VK_3]        = 3;    keyBit[KeyEvent.VK_3]        = 0x04;
        keyRow[KeyEvent.VK_4]        = 3;    keyBit[KeyEvent.VK_4]        = 0x08;
        keyRow[KeyEvent.VK_5]        = 3;    keyBit[KeyEvent.VK_5]        = 0x10;
        keyRow[KeyEvent.VK_0]        = 4;    keyBit[KeyEvent.VK_0]        = 0x01;
        keyRow[KeyEvent.VK_9]        = 4;    keyBit[KeyEvent.VK_9]        = 0x02;
        keyRow[KeyEvent.VK_8]        = 4;    keyBit[KeyEvent.VK_8]        = 0x04;
        keyRow[KeyEvent.VK_7]        = 4;    keyBit[KeyEvent.VK_7]        = 0x08;
        keyRow[KeyEvent.VK_6]        = 4;    keyBit[KeyEvent.VK_6]        = 0x10;
        keyRow[KeyEvent.VK_P]        = 5;    keyBit[KeyEvent.VK_P]        = 0x01;
        keyRow[KeyEvent.VK_O]        = 5;    keyBit[KeyEvent.VK_O]        = 0x02;
        keyRow[KeyEvent.VK_I]        = 5;    keyBit[KeyEvent.VK_I]        = 0x04;
        keyRow[KeyEvent.VK_U]        = 5;    keyBit[KeyEvent.VK_U]        = 0x08;
        keyRow[KeyEvent.VK_Y]        = 5;    keyBit[KeyEvent.VK_Y]        = 0x10;
        keyRow[KeyEvent.VK_ENTER]    = 6;    keyBit[KeyEvent.VK_ENTER]    = 0x01;
        keyRow[KeyEvent.VK_L]        = 6;    keyBit[KeyEvent.VK_L]        = 0x02;
        keyRow[KeyEvent.VK_K]        = 6;    keyBit[KeyEvent.VK_K]        = 0x04;
        keyRow[KeyEvent.VK_J]        = 6;    keyBit[KeyEvent.VK_J]        = 0x08;
        keyRow[KeyEvent.VK_H]        = 6;    keyBit[KeyEvent.VK_H]        = 0x10;
        keyRow[KeyEvent.VK_SPACE]    = 7;    keyBit[KeyEvent.VK_SPACE]    = 0x01;
        keyRow[KeyEvent.VK_CONTROL]  = 7;    keyBit[KeyEvent.VK_CONTROL]  = 0x02;
        keyRow[KeyEvent.VK_M]        = 7;    keyBit[KeyEvent.VK_M]        = 0x04;
        keyRow[KeyEvent.VK_N]        = 7;    keyBit[KeyEvent.VK_N]        = 0x08;
        keyRow[KeyEvent.VK_B]        = 7;    keyBit[KeyEvent.VK_B]        = 0x10;
        // allow ALT for symbol shift too
        keyRow[KeyEvent.VK_ALT]      = 7;    keyBit[KeyEvent.VK_ALT]      = 0x02;
    }
    
    public void keyTyped(KeyEvent e) {
        // ignore keyTyped events, they're *far* too high level
    }
    
    public void keyPressed(KeyEvent key) {
        int keyCode = key.getKeyCode();
        if (keyCode < 256) {
            this.keyRowState[keyRow[keyCode]] &= ~(keyBit[keyCode]);
        }
    }
    
    public void keyReleased(KeyEvent key) {
        int keyCode = key.getKeyCode();
        if (keyCode < 256) {
            this.keyRowState[keyRow[keyCode]] |= keyBit[keyCode];
        }
    }
    
    public int readPort(int port) {
        int result = 0x1F;
        for (int i=0; i < 8; i++) {
            if ((port & (0x0100 << i)) == 0) {
                result &= keyRowState[i];
            }
        }
        return result;
    }
}
